"""Test calendar rrules."""

import datetime

from django.test import TestCase

from dateutil import tz
from dateutil.rrule import DAILY
from dateutil.rrule import MO
from dateutil.rrule import MONTHLY
from dateutil.rrule import TH
from dateutil.rrule import TU
from dateutil.rrule import WEEKLY
from dateutil.rrule import YEARLY
from dateutil.rrule import rrule
from icalendar.prop import vRecur
import recurrence

from django_ical import utils


class BuildRruleTest(TestCase):
    """Test building an Rrule for icalendar."""

    def test_every_day(self):
        """Repeat every day."""
        vrecurr = vRecur(utils.build_rrule(freq="DAILY"))
        assert vrecurr["FREQ"] == "DAILY"
        assert vrecurr.to_ical().decode() == "FREQ=DAILY"
        assert len(vrecurr.keys()) == 1

    def test_daily_byhour(self):
        """Repeat every day at 10, 12 and 17."""
        vrecurr = utils.build_rrule(freq="DAILY", byhour=[10, 12, 17])
        assert vrecurr["FREQ"] == "DAILY"
        assert vrecurr["BYHOUR"] == [10, 12, 17]
        vRecur(vrecurr).to_ical().decode() == "FREQ=DAILY;BYHOUR=10,12,17"
        assert len(vrecurr.keys()) == 2

    def test_daily_byhour_once(self):
        """Repeat every day at 10."""
        vrecurr = utils.build_rrule(freq="DAILY", byhour=10)
        assert vrecurr["FREQ"] == "DAILY"
        assert vrecurr["BYHOUR"] == 10
        vRecur(vrecurr).to_ical().decode() == "FREQ=DAILY;BYHOUR=10"
        assert len(vrecurr.keys()) == 2

    def test_every_week(self):
        """Repeat every week."""
        vrecurr = utils.build_rrule(freq="WEEKLY")
        assert vrecurr["FREQ"] == "WEEKLY"
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY"
        assert len(vrecurr.keys()) == 1

    def test_ever_hour(self):
        """Repeat every hour."""
        vrecurr = utils.build_rrule(freq="HOURLY")
        assert vrecurr["FREQ"] == "HOURLY"
        vRecur(vrecurr).to_ical().decode() == "FREQ=HOURLY"
        assert len(vrecurr.keys()) == 1

    def test_ever_4_hours(self):
        """Repeat every 4 hours."""
        vrecurr = utils.build_rrule(interval=4, freq="HOURLY")
        assert vrecurr["FREQ"] == "HOURLY"
        assert vrecurr["INTERVAL"] == 4
        vRecur(vrecurr).to_ical().decode() == "FREQ=HOURLY;INTERVAL=4"
        assert len(vrecurr.keys()) == 2

    def test_weekly_tue(self):
        """Repeat every week on Tuesday."""
        vrecurr = utils.build_rrule(freq="WEEKLY", byday="TU")
        assert vrecurr["FREQ"] == "WEEKLY"
        assert vrecurr["BYDAY"] == "TU"
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;BYDAY=TU"
        assert len(vrecurr.keys()) == 2

    def test_weekly_mo_wed(self):
        """Repeat every week on Monday, Wednesday."""
        vrecurr = utils.build_rrule(freq="WEEKLY", byday=["MO", "WE"])
        assert vrecurr["FREQ"] == "WEEKLY"
        assert vrecurr["BYDAY"] == ["MO", "WE"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;BYDAY=MO,WE"
        assert len(vrecurr.keys()) == 2

    def test_every_weekday(self):
        """Repeat every weekday."""
        vrecurr = utils.build_rrule(freq="WEEKLY", byday=["MO", "TU", "WE", "TH", "FR"])
        assert vrecurr["FREQ"] == "WEEKLY"
        assert vrecurr["BYDAY"] == ["MO", "TU", "WE", "TH", "FR"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR"
        assert len(vrecurr.keys()) == 2

    def test_every_2_weeks(self):
        """Repeat every 2 weeks."""
        vrecurr = utils.build_rrule(interval=2, freq="WEEKLY")
        assert vrecurr["FREQ"] == "WEEKLY"
        assert vrecurr["INTERVAL"] == 2
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;INTERVAL=2"
        assert len(vrecurr.keys()) == 2

    def test_every_month(self):
        """Repeat every month."""
        vrecurr = utils.build_rrule(freq="MONTHLY")
        assert vrecurr["FREQ"] == "MONTHLY"
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY"
        assert len(vrecurr.keys()) == 1

    def test_every_6_months(self):
        """Repeat very 6 months."""
        vrecurr = utils.build_rrule(interval=6, freq="MONTHLY")
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["INTERVAL"] == 6
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;INTERVAL=6"
        assert len(vrecurr.keys()) == 2

    def test_every_year(self):
        """Repeat every year."""
        vrecurr = utils.build_rrule(freq="YEARLY")
        assert vrecurr["FREQ"] == "YEARLY"
        vRecur(vrecurr).to_ical().decode() == "FREQ=YEARLY"
        assert len(vrecurr.keys()) == 1

    def test_every_month_on_the_4th(self):
        """Repeat every month on the 4th."""
        vrecurr = utils.build_rrule(freq="MONTHLY", bymonthday=4)
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYMONTHDAY"] == 4
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=4"
        assert len(vrecurr.keys()) == 2

    def test_every_month_on_the_4th_last(self):
        """Repeat every month on the 4th last."""
        vrecurr = utils.build_rrule(freq="MONTHLY", bymonthday=-4)
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYMONTHDAY"] == -4
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=-4"
        assert len(vrecurr.keys()) == 2

    def test_ever_month_3rd_tu(self):
        """Repeat every month on the 3rd Tuesday."""
        vrecurr = utils.build_rrule(freq="MONTHLY", byday="+3TU")
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYDAY"] == "+3TU"
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=+3TU"
        assert len(vrecurr.keys()) == 2

    def test_ever_month_3rd_last_tu(self):
        """Repeat every month on the 3rd last Tuesday."""
        vrecurr = utils.build_rrule(freq="MONTHLY", byday="-3TU")
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYDAY"] == "-3TU"
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=-3TU"
        assert len(vrecurr.keys()) == 2

    def test_ever_month_last_mo(self):
        """Repeat every month on the last Monday."""
        vrecurr = utils.build_rrule(freq="MONTHLY", byday="-1MO")
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYDAY"] == "-1MO"
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=-1MO"
        assert len(vrecurr.keys()) == 2

    def test_every_week_until_jan_2007(self):
        """Repeat every week until January 1, 2007."""
        utc = tz.UTC
        jan2007 = datetime.datetime(2007, 1, 1, 0, 0, tzinfo=utc)
        vrecurr = utils.build_rrule(freq="WEEKLY", until=jan2007)
        assert vrecurr["FREQ"] == "WEEKLY"
        assert vrecurr["UNTIL"] == jan2007
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;UNTIL=20070101T000000Z"
        assert len(vrecurr.keys()) == 2

    def test_every_week_20_times(self):
        """Repeat every week for 20 times."""
        vrecurr = utils.build_rrule(freq="WEEKLY", count=20)
        assert vrecurr["FREQ"] == "WEEKLY"
        assert vrecurr["COUNT"] == 20
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;COUNT=20"
        assert len(vrecurr.keys()) == 2

    def test_every_month_last_working_day(self):
        """Repeat the last working day of each month."""
        vrecurr = utils.build_rrule(
            freq="MONTHLY", byday=["MO", "TU", "WE", "TH", "FR"], bysetpos=-1
        )
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYDAY"] == ["MO", "TU", "WE", "TH", "FR"]
        assert vrecurr["BYSETPOS"] == -1
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=-1"
        assert len(vrecurr.keys()) == 3

    def test_ever_month_last_day(self):
        """Repeat the last day of each month."""
        vrecurr = utils.build_rrule(freq="MONTHLY", bymonthday=-1)
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYMONTHDAY"] == -1
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=-1"
        assert len(vrecurr.keys()) == 2

    def test_every_day_in_jan(self):
        """Repeat every day in January"""
        vrecurr = utils.build_rrule(
            freq="YEARLY", bymonth=1, byday=["MO", "TU", "WE", "TH", "FR", "SA", "SU"]
        )
        assert vrecurr["FREQ"] == "YEARLY"
        assert vrecurr["BYMONTH"] == 1
        assert vrecurr["BYDAY"] == ["MO", "TU", "WE", "TH", "FR", "SA", "SU"]
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=YEARLY;BYDAY=MO,TU,WE,TH,FR,SA,SU;BYMONTH=1"
        assert len(vrecurr.keys()) == 3

    def test_every_2nd_15th_of_month(self):
        """Repeat monthly on the 2nd and 15th of the month."""
        vrecurr = utils.build_rrule(freq="MONTHLY", bymonthday=[4, 15])
        assert vrecurr["FREQ"] == "MONTHLY"
        assert vrecurr["BYMONTHDAY"] == [4, 15]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=4,15"
        assert len(vrecurr.keys()) == 2

    def test_every_fr_13th(self):
        """Repeat every Friday the 13th."""
        vrecurr = utils.build_rrule(freq="YEARLY", bymonthday=13, byday="FR")
        assert vrecurr["FREQ"] == "YEARLY"
        assert vrecurr["BYMONTHDAY"] == 13
        assert vrecurr["BYDAY"] == "FR"
        vRecur(vrecurr).to_ical().decode() == "FREQ=YEARLY;BYDAY=FR;BYMONTHDAY=13"
        assert len(vrecurr.keys()) == 3


class FromTextTests(TestCase):
    """Test build a vRecur dictionary from an RRULE string."""

    def test_every_day(self):
        """Repeat every day."""
        vrecurr = utils.build_rrule_from_text("FREQ=DAILY")
        assert vrecurr["FREQ"] == ["DAILY"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=DAILY"
        assert len(vrecurr.keys()) == 1

    def test_daily_byhour(self):
        """Repeat every day at 10, 12 and 17."""
        vrecurr = utils.build_rrule_from_text("FREQ=DAILY;BYHOUR=10,12,17")
        assert vrecurr["FREQ"] == ["DAILY"]
        assert vrecurr["BYHOUR"] == [10, 12, 17]
        vRecur(vrecurr).to_ical().decode() == "FREQ=DAILY;BYHOUR=10,12,17"
        assert len(vrecurr.keys()) == 2

    def test_every_week(self):
        """Repeat every week."""
        vrecurr = utils.build_rrule_from_text("FREQ=WEEKLY")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY"
        assert len(vrecurr.keys()) == 1

    def test_ever_hour(self):
        """Repeat every hour."""
        vrecurr = utils.build_rrule_from_text("FREQ=HOURLY")
        assert vrecurr["FREQ"] == ["HOURLY"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=HOURLY"
        assert len(vrecurr.keys()) == 1

    def test_ever_4_hours(self):
        """Repeat every 4 hours."""
        vrecurr = utils.build_rrule_from_text("INTERVAL=4;FREQ=HOURLY")
        assert vrecurr["FREQ"] == ["HOURLY"]
        assert vrecurr["INTERVAL"] == [4]
        vRecur(vrecurr).to_ical().decode() == "FREQ=HOURLY;INTERVAL=4"
        assert len(vrecurr.keys()) == 2

    def test_weekly_tue(self):
        """Repeat every week on Tuesday."""
        vrecurr = utils.build_rrule_from_text("FREQ=WEEKLY;BYDAY=TU")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        assert vrecurr["BYDAY"] == ["TU"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;BYDAY=TU"
        assert len(vrecurr.keys()) == 2

    def test_weekly_mo_wed(self):
        """Repeat every week on Monday, Wednesday."""
        vrecurr = utils.build_rrule_from_text("FREQ=WEEKLY;BYDAY=MO,WE")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        assert vrecurr["BYDAY"] == ["MO", "WE"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;BYDAY=MO,WE"
        assert len(vrecurr.keys()) == 2

    def test_every_weekday(self):
        """Repeat every weekday."""
        vrecurr = utils.build_rrule_from_text("FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        assert vrecurr["BYDAY"] == ["MO", "TU", "WE", "TH", "FR"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR"
        assert len(vrecurr.keys()) == 2

    def test_every_2_weeks(self):
        """Repeat every 2 weeks."""
        vrecurr = utils.build_rrule_from_text("INTERVAL=2;FREQ=WEEKLY")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        assert vrecurr["INTERVAL"] == [2]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;INTERVAL=2"
        assert len(vrecurr.keys()) == 2

    def test_every_month(self):
        """Repeat every month."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY"
        assert len(vrecurr.keys()) == 1

    def test_every_6_months(self):
        """Repeat very 6 months."""
        vrecurr = utils.build_rrule_from_text("INTERVAL=6;FREQ=MONTHLY")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["INTERVAL"] == [6]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;INTERVAL=6"
        assert len(vrecurr.keys()) == 2

    def test_every_year(self):
        """Repeat every year."""
        vrecurr = utils.build_rrule_from_text("FREQ=YEARLY")
        assert vrecurr["FREQ"] == ["YEARLY"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=YEARLY"
        assert len(vrecurr.keys()) == 1

    def test_every_month_on_the_4th(self):
        """Repeat every month on the 4th."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYMONTHDAY=4")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYMONTHDAY"] == [4]
        assert len(vrecurr.keys()) == 2
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYMONTHDAY=+4")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYMONTHDAY"] == [4]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=4"
        assert len(vrecurr.keys()) == 2

    def test_every_month_on_the_4th_last(self):
        """Repeat every month on the 4th last."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYMONTHDAY=-4")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYMONTHDAY"] == [-4]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=-4"
        assert len(vrecurr.keys()) == 2

    def test_ever_month_3rd_tu(self):
        """Repeat every month on the 3rd Tuesday."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYDAY=+3TU")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYDAY"] == ["+3TU"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=+3TU"
        assert len(vrecurr.keys()) == 2

    def test_ever_month_3rd_last_tu(self):
        """Repeat every month on the 3rd last Tuesday."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYDAY=-3TU")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYDAY"] == ["-3TU"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=-3TU"
        assert len(vrecurr.keys()) == 2

    def test_ever_month_last_mo(self):
        """Repeat every month on the last Monday."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYDAY=-1MO")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYDAY"] == ["-1MO"]
        assert len(vrecurr.keys()) == 2

    def test_ever_month_second_last_fr(self):
        """Repeat every month on the 2nd last Friday."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYDAY=-2FR")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYDAY"] == ["-2FR"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=-2FR"
        assert len(vrecurr.keys()) == 2

    def test_every_week_until_jan_2007(self):
        """Repeat every week until January 1, 2007."""
        utc = tz.UTC
        vrecurr = utils.build_rrule_from_text("FREQ=WEEKLY;UNTIL=20070101T000000Z")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        assert vrecurr["UNTIL"] == [datetime.datetime(2007, 1, 1, 0, 0, tzinfo=utc)]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;UNTIL=20070101T000000Z"
        assert len(vrecurr.keys()) == 2

    def test_every_week_20_times(self):
        """Repeat every week for 20 times."""
        vrecurr = utils.build_rrule_from_text("FREQ=WEEKLY;COUNT=20")
        assert vrecurr["FREQ"] == ["WEEKLY"]
        assert vrecurr["COUNT"] == [20]
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;COUNT=20"
        assert len(vrecurr.keys()) == 2

    def test_every_month_last_working_day(self):
        """Repeat the last working day of each month."""
        vrecurr = utils.build_rrule_from_text(
            "FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=-1;"
        )
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYDAY"] == ["MO", "TU", "WE", "TH", "FR"]
        assert vrecurr["BYSETPOS"] == [-1]
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=MONTHLY;BYDAY=MO,TU,WE,TH,FR;BYSETPOS=-1"
        assert len(vrecurr.keys()) == 3

    def test_ever_month_last_day(self):
        """Repeat the last day of each month."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYMONTHDAY=-1")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYMONTHDAY"] == [-1]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=-1"
        assert len(vrecurr.keys()) == 2

    def test_every_day_in_jan(self):
        """Repeat every day in January"""
        vrecurr = utils.build_rrule_from_text(
            "FREQ=YEARLY;BYMONTH=1;BYDAY=MO,TU,WE,TH,FR,SA,SU;"
        )
        assert vrecurr["FREQ"] == ["YEARLY"]
        assert vrecurr["BYMONTH"] == [1]
        assert vrecurr["BYDAY"] == ["MO", "TU", "WE", "TH", "FR", "SA", "SU"]
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=YEARLY;BYDAY=MO,TU,WE,TH,FR,SA,SU;BYMONTH=1"
        assert len(vrecurr.keys()) == 3

    def test_every_2nd_15th_of_month(self):
        """Repeat monthly on the 2nd and 15th of the month."""
        vrecurr = utils.build_rrule_from_text("FREQ=MONTHLY;BYMONTHDAY=4,15")
        assert vrecurr["FREQ"] == ["MONTHLY"]
        assert vrecurr["BYMONTHDAY"] == [4, 15]
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYMONTHDAY=4,15"
        assert len(vrecurr.keys()) == 2

    def test_every_fr_13th(self):
        """Repeat every Friday the 13th."""
        vrecurr = utils.build_rrule_from_text("FREQ=YEARLY;BYMONTHDAY=13;BYDAY=FR")
        assert vrecurr["FREQ"] == ["YEARLY"]
        assert vrecurr["BYMONTHDAY"] == [13]
        assert vrecurr["BYDAY"] == ["FR"]
        vRecur(vrecurr).to_ical().decode() == "FREQ=YEARLY;BYDAY=FR;BYMONTHDAY=13"
        assert len(vrecurr.keys()) == 3


class FromDateutilRruleTests(TestCase):
    """Build an ical string from a dateutil rrule."""

    def test_weekly_by_month_year_day(self):
        rule = rrule(
            WEEKLY,
            bymonth=(1, 7),
            byyearday=(1, 100, 200, 365),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=WEEKLY;BYYEARDAY=1,100,200,365;BYMONTH=1,7"

    def test_weekly_by_month_nweekday(self):
        rule = rrule(
            WEEKLY,
            count=3,
            bymonth=(1, 3),
            byweekday=(TU(1), TH(-1)),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=WEEKLY;COUNT=3;BYDAY=TU,TH;BYMONTH=1,3"

    def test_weekly_by_monthday(self):
        rule = rrule(
            WEEKLY,
            count=3,
            bymonthday=(1, 3),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;COUNT=3;BYMONTHDAY=1,3"

    def test_weekly_by_weekday(self):
        rule = rrule(
            WEEKLY,
            count=3,
            byweekday=(TU, TH),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY;COUNT=3;BYDAY=TU,TH"

    def test_daily_by_month_nweekday(self):
        rule = rrule(
            DAILY,
            count=3,
            bymonth=(1, 3),
            byweekday=(TU(1), TH(-1)),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=DAILY;COUNT=3;BYDAY=TU,TH;BYMONTH=1,3"

    def test_yearly_month_nweekday(self):
        rule = rrule(
            YEARLY,
            count=3,
            bymonth=(1, 3),
            byweekday=(TU(1), TH(-1)),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=YEARLY;COUNT=3;BYDAY=+1TU,-1TH;BYMONTH=1,3"

    def test_yearly_month_yearday(self):
        rule = rrule(
            YEARLY,
            count=4,
            bymonth=(4, 7),
            byyearday=(1, 100, 200, 365),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        assert (
            vRecur(vrecurr).to_ical().decode()
            == "FREQ=YEARLY;COUNT=4;BYYEARDAY=1,100,200,365;BYMONTH=4,7"
        )

    def test_yearly_weekno_weekday(self):
        rule = rrule(
            YEARLY,
            count=3,
            byweekno=1,
            byweekday=MO,
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(vrecurr).to_ical().decode() == "FREQ=YEARLY;COUNT=3;BYDAY=MO;BYWEEKNO=1"

    def test_yearly_setpos(self):
        rule = rrule(
            YEARLY,
            count=3,
            bymonthday=15,
            byhour=(6, 18),
            bysetpos=(3, -3),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        assert (
            vRecur(vrecurr).to_ical().decode()
            == "FREQ=YEARLY;COUNT=3;BYHOUR=6,18;BYMONTHDAY=15;BYSETPOS=3,-3"
        )

    def test_monthly_month_monthday(self):
        rule = rrule(
            MONTHLY,
            count=3,
            bymonth=(1, 3),
            bymonthday=(5, 7),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(
            vrecurr
        ).to_ical().decode() == "FREQ=MONTHLY;COUNT=3;BYMONTHDAY=5,7;BYMONTH=1,3"

    def test_monthly_nweekday(self):
        rule = rrule(
            MONTHLY,
            count=3,
            byweekday=(TU(1), TH(-1)),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;COUNT=3;BYDAY=+1TU,-1TH"

    def test_monthly_month_nweekday(self):
        rule = rrule(
            MONTHLY,
            bymonth=(1, 3),
            byweekday=(TU(1), TH(-1)),
            dtstart=datetime.datetime(1997, 9, 2, 9, 0),
        )
        vrecurr = utils.build_rrule_from_dateutil_rrule(rule)
        vRecur(vrecurr).to_ical().decode() == "FREQ=MONTHLY;BYDAY=+1TU,-1TH;BYMONTH=1,3"


class FromDjangoRecurrenceRruleTests(TestCase):
    """Build an ical string from a django-recurrence rrule."""

    def test_rule(self):
        rule = recurrence.Rule(recurrence.WEEKLY)
        vrecurr = utils.build_rrule_from_recurrences_rrule(rule)
        vRecur(vrecurr).to_ical().decode() == "FREQ=WEEKLY"

    def test_complex_rule_serialization(self):
        rule = recurrence.Rule(
            recurrence.WEEKLY,
            interval=17,
            wkst=1,
            count=7,
            byday=[recurrence.to_weekday("-1MO"), recurrence.to_weekday("TU")],
            bymonth=[1, 3],
        )
        vrecurr = utils.build_rrule_from_recurrences_rrule(rule)
        assert (
            vRecur(vrecurr).to_ical().decode()
            == "FREQ=WEEKLY;COUNT=7;INTERVAL=17;BYDAY=-1MO,TU;BYMONTH=1,3;WKST=TU"
        )

    def test_complex_rule_serialization_with_weekday_instance(self):
        rule = recurrence.Rule(
            recurrence.WEEKLY,
            interval=17,
            wkst=recurrence.to_weekday(1),
            count=7,
            byday=[recurrence.to_weekday("-1MO"), recurrence.to_weekday("TU")],
            bymonth=[1, 3],
        )
        vrecurr = utils.build_rrule_from_recurrences_rrule(rule)
        assert (
            vRecur(vrecurr).to_ical().decode()
            == "FREQ=WEEKLY;COUNT=7;INTERVAL=17;BYDAY=-1MO,TU;BYMONTH=1,3;WKST=TU"
        )



from datetime import date
from datetime import datetime
from datetime import timedelta
from os import linesep

from django.test import TestCase
from django.test.client import RequestFactory

from dateutil import tz
import icalendar

from django_ical import utils
from django_ical.feedgenerator import ICal20Feed
from django_ical.views import ICalFeed


class TestICalFeed(ICalFeed):
    feed_type = ICal20Feed
    title = "Test Feed"
    description = "Test ICal Feed"
    items = []


class TestItemsFeed(ICalFeed):
    feed_type = ICal20Feed
    title = "Test Feed"
    description = "Test ICal Feed"

    def items(self):
        return [
            {
                "title": "Title1",
                "description": "Description1",
                "link": "/event/1",
                "start": datetime(2012, 5, 1, 18, 0),
                "end": datetime(2012, 5, 1, 20, 0),
                "recurrences": {
                    "rrules": [
                        utils.build_rrule(freq="DAILY", byhour=10),
                        utils.build_rrule(freq="MONTHLY", bymonthday=4),
                    ],
                    "xrules": [
                        utils.build_rrule(freq="MONTHLY", bymonthday=-4),
                        utils.build_rrule(freq="MONTHLY", byday="+3TU"),
                    ],
                    "rdates": [date(1999, 9, 2), date(1998, 1, 1)],
                    "xdates": [date(1999, 8, 1), date(1998, 2, 1)],
                },
                "geolocation": (37.386013, -122.082932),
                "organizer": "john.doe@example.com",
                "participants": [
                    {
                        "email": "joe.unresponsive@example.com",
                        "cn": "Joe Unresponsive",
                        "partstat": "NEEDS-ACTION",
                    },
                    {
                        "email": "jane.attender@example.com",
                        "cn": "Jane Attender",
                        "partstat": "ACCEPTED",
                    },
                    {
                        "email": "dan.decliner@example.com",
                        "cn": "Dan Decliner",
                        "partstat": "DECLINED",
                    },
                    {
                        "email": "mary.maybe@example.com",
                        "cn": "Mary Maybe",
                        "partstat": "TENTATIVE",
                    },
                ],
                "modified": datetime(2012, 5, 2, 10, 0),
            },
            {
                "title": "Title2",
                "description": "Description2",
                "link": "/event/2",
                "start": datetime(2012, 5, 6, 18, 0),
                "end": datetime(2012, 5, 6, 20, 0),
                "recurrences": {
                    "rrules": [
                        utils.build_rrule(
                            freq="WEEKLY", byday=["MO", "TU", "WE", "TH", "FR"]
                        )
                    ],
                    "xrules": [utils.build_rrule(freq="MONTHLY", byday="-3TU")],
                    "rdates": [date(1997, 9, 2)],
                    "xdates": [date(1997, 8, 1)],
                },
                "geolocation": (37.386013, -122.082932),
                "modified": datetime(2012, 5, 7, 10, 0),
                "organizer": {
                    "cn": "John Doe",
                    "email": "john.doe@example.com",
                    "role": "CHAIR",
                },
                "alarms": [
                    {
                        "trigger": timedelta(minutes=-30),
                        "action": "DISPLAY",
                        "description": "Alarm2a",
                    },
                    {
                        "trigger": timedelta(days=-1),
                        "action": "DISPLAY",
                        "description": "Alarm2b",
                    },
                ],
            },
        ]

    def item_title(self, obj):
        return obj["title"]

    def item_description(self, obj):
        return obj["description"]

    def item_start_datetime(self, obj):
        return obj["start"]

    def item_end_datetime(self, obj):
        return obj["end"]

    def item_rrule(self, obj):
        return obj["recurrences"]["rrules"]

    def item_exrule(self, obj):
        return obj["recurrences"]["xrules"]

    def item_rdate(self, obj):
        return obj["recurrences"]["rdates"]

    def item_exdate(self, obj):
        return obj["recurrences"]["xdates"]

    def item_link(self, obj):
        return obj["link"]

    def item_geolocation(self, obj):
        return obj.get("geolocation", None)

    def item_updateddate(self, obj):
        return obj.get("modified", None)

    def item_pubdate(self, obj):
        return obj.get("modified", None)

    def item_organizer(self, obj):
        organizer_dic = obj.get("organizer", None)
        if organizer_dic:
            if isinstance(organizer_dic, dict):
                organizer = icalendar.vCalAddress("MAILTO:%s" % organizer_dic["email"])
                for key, val in organizer_dic.items():
                    if key != "email":
                        organizer.params[key] = icalendar.vText(val)
            else:
                organizer = icalendar.vCalAddress("MAILTO:%s" % organizer_dic)
            return organizer

    def item_attendee(self, obj):
        """All calendars support ATTENDEE attribute, however, at this time, Apple calendar (desktop & iOS) and Outlook
        display event attendees, while Google does not. For SUBSCRIBED calendars it seems that it is not possible to
        use the default method to respond. As an alternative, you may review adding custom links to your description
        or setting up something like CalDav with authentication, which can enable the ability for attendees to respond
        via the default icalendar protocol."""
        participants = obj.get("participants", None)
        if participants:
            attendee_list = list()
            default_attendee_params = {
                "cutype": icalendar.vText("INDIVIDUAL"),
                "role": icalendar.vText("REQ-PARTICIPANT"),
                "rsvp": icalendar.vText(
                    "TRUE"
                ),  # Does not seem to work for subscribed calendars.
            }
            for participant in participants:
                attendee = icalendar.vCalAddress("MAILTO:%s" % participant.pop("email"))
                participant_dic = default_attendee_params.copy()
                participant_dic.update(participant)
                for key, val in participant_dic.items():
                    attendee.params[key] = icalendar.vText(val)
                attendee_list.append(attendee)
            return attendee_list

    def item_valarm(self, obj):
        alarms = obj.get("alarms", None)
        if alarms:
            alarm_list = list()
            for alarm in alarms:
                valarm = icalendar.Alarm()
                for key, value in alarm.items():
                    valarm.add(key, value)
                alarm_list.append(valarm)
            return alarm_list


class TestFilenameFeed(ICalFeed):
    feed_type = ICal20Feed
    title = "Test Filename Feed"
    description = "Test ICal Feed"

    def get_object(self, request):
        return {"id": 123}

    def items(self, obj):
        return [obj]

    def file_name(self, obj):
        return "%s.ics" % obj["id"]

    def item_link(self, item):
        return ""  # Required by the syndication framework


class ICal20FeedTest(TestCase):
    def test_basic(self):
        request = RequestFactory().get("/test/ical")
        view = TestICalFeed()

        response = view(request)
        calendar = icalendar.Calendar.from_ical(response.content)
        self.assertEqual(calendar["X-WR-CALNAME"], "Test Feed")
        self.assertEqual(calendar["X-WR-CALDESC"], "Test ICal Feed")

    def test_items(self):
        request = RequestFactory().get("/test/ical")
        view = TestItemsFeed()

        response = view(request)

        calendar = icalendar.Calendar.from_ical(response.content)
        self.assertEqual(len(calendar.subcomponents), 2)

        self.assertEqual(calendar.subcomponents[0]["SUMMARY"], "Title1")
        self.assertEqual(calendar.subcomponents[0]["DESCRIPTION"], "Description1")
        self.assertTrue(calendar.subcomponents[0]["URL"].endswith("/event/1"))
        self.assertEqual(
            calendar.subcomponents[0]["DTSTART"].to_ical(), b"20120501T180000"
        )
        self.assertEqual(
            calendar.subcomponents[0]["DTEND"].to_ical(), b"20120501T200000"
        )
        self.assertEqual(
            calendar.subcomponents[0]["GEO"].to_ical(), "37.386013;-122.082932"
        )
        self.assertEqual(
            calendar.subcomponents[0]["LAST-MODIFIED"].to_ical(), b"20120502T100000Z"
        )
        self.assertEqual(
            calendar.subcomponents[0]["ORGANIZER"].to_ical(),
            b"MAILTO:john.doe@example.com",
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][0].to_ical(),
            b"MAILTO:joe.unresponsive@example.com",
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][0].params.to_ical(),
            b'CN="Joe Unresponsive";CUTYPE=INDIVIDUAL;PARTSTAT=NEEDS-ACTION;ROLE=REQ-PARTICIPANT;'
            b"RSVP=TRUE",
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][1].to_ical(),
            b"MAILTO:jane.attender@example.com",
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][1].params.to_ical(),
            b'CN="Jane Attender";CUTYPE=INDIVIDUAL;PARTSTAT=ACCEPTED;ROLE=REQ-PARTICIPANT;RSVP=TRUE',
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][2].to_ical(),
            b"MAILTO:dan.decliner@example.com",
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][2].params.to_ical(),
            b'CN="Dan Decliner";CUTYPE=INDIVIDUAL;PARTSTAT=DECLINED;ROLE=REQ-PARTICIPANT;RSVP=TRUE',
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][3].to_ical(),
            b"MAILTO:mary.maybe@example.com",
        )
        self.assertEqual(
            calendar.subcomponents[0]["ATTENDEE"][3].params.to_ical(),
            b'CN="Mary Maybe";CUTYPE=INDIVIDUAL;PARTSTAT=TENTATIVE;ROLE=REQ-PARTICIPANT;RSVP=TRUE',
        )
        self.assertEqual(
            calendar.subcomponents[0]["RRULE"][0].to_ical(), b"FREQ=DAILY;BYHOUR=10"
        )
        self.assertEqual(
            calendar.subcomponents[0]["RRULE"][1].to_ical(),
            b"FREQ=MONTHLY;BYMONTHDAY=4",
        )
        self.assertEqual(
            calendar.subcomponents[0]["EXRULE"][0].to_ical(),
            b"FREQ=MONTHLY;BYMONTHDAY=-4",
        )
        self.assertEqual(
            calendar.subcomponents[0]["EXRULE"][1].to_ical(), b"FREQ=MONTHLY;BYDAY=+3TU"
        )
        self.assertEqual(
            calendar.subcomponents[0]["RDATE"].to_ical(), b"19990902,19980101"
        )
        self.assertEqual(
            calendar.subcomponents[0]["EXDATE"].to_ical(), b"19990801,19980201"
        )

        self.assertEqual(calendar.subcomponents[1]["SUMMARY"], "Title2")
        self.assertEqual(calendar.subcomponents[1]["DESCRIPTION"], "Description2")
        self.assertTrue(calendar.subcomponents[1]["URL"].endswith("/event/2"))
        self.assertEqual(
            calendar.subcomponents[1]["DTSTART"].to_ical(), b"20120506T180000"
        )
        self.assertEqual(
            calendar.subcomponents[1]["DTEND"].to_ical(), b"20120506T200000"
        )
        self.assertEqual(
            calendar.subcomponents[1]["RRULE"].to_ical(),
            b"FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR",
        )
        self.assertEqual(
            calendar.subcomponents[1]["EXRULE"].to_ical(), b"FREQ=MONTHLY;BYDAY=-3TU"
        )
        self.assertEqual(calendar.subcomponents[1]["RDATE"].to_ical(), b"19970902")
        self.assertEqual(calendar.subcomponents[1]["EXDATE"].to_ical(), b"19970801")
        self.assertEqual(
            calendar.subcomponents[1]["GEO"].to_ical(), "37.386013;-122.082932"
        )
        self.assertEqual(
            calendar.subcomponents[1]["LAST-MODIFIED"].to_ical(), b"20120507T100000Z"
        )
        self.assertEqual(
            calendar.subcomponents[1]["ORGANIZER"].to_ical(),
            b"MAILTO:john.doe@example.com",
        )
        self.assertIn(
            b"BEGIN:VALARM\r\nACTION:DISPLAY\r\nDESCRIPTION:Alarm2a\r\nTRIGGER:-PT30M\r\nEND:VALARM\r\n",
            [comp.to_ical() for comp in calendar.subcomponents[1].subcomponents],
        )
        self.assertIn(
            b"BEGIN:VALARM\r\nACTION:DISPLAY\r\nDESCRIPTION:Alarm2b\r\nTRIGGER:-P1D\r\nEND:VALARM\r\n",
            [comp.to_ical() for comp in calendar.subcomponents[1].subcomponents],
        )

    def test_wr_timezone(self):
        """
        Test for the x-wr-timezone property.
        """

        class TestTimezoneFeed(TestICalFeed):
            timezone = "Asia/Tokyo"

        request = RequestFactory().get("/test/ical")
        view = TestTimezoneFeed()

        response = view(request)
        calendar = icalendar.Calendar.from_ical(response.content)
        self.assertEqual(calendar["X-WR-TIMEZONE"], "Asia/Tokyo")

    def test_timezone(self):
        tokyo = tz.gettz("Asia/Tokyo")  # also known as JST or Japan Standard Time
        us_eastern = tz.gettz("US/Eastern")  # also known as EDT or Eastern (Daylight) Time

        class TestTimezoneFeed(TestItemsFeed):
            def items(self):
                return [
                    {
                        "title": "Title1",
                        "description": "Description1",
                        "link": "/event/1",
                        "start": datetime(2012, 5, 1, 18, 00, tzinfo=tokyo),
                        "end": datetime(2012, 5, 1, 20, 00, tzinfo=tokyo),
                        "recurrences": {
                            "rrules": [],
                            "xrules": [],
                            "rdates": [],
                            "xdates": [],
                        },
                    },
                    {
                        "title": "Title2",
                        "description": "Description2",
                        "link": "/event/2",
                        "start": datetime(2012, 5, 6, 18, 00, tzinfo=us_eastern),
                        "end": datetime(2012, 5, 6, 20, 00, tzinfo=us_eastern),
                        "recurrences": {
                            "rrules": [],
                            "xrules": [],
                            "rdates": [],
                            "xdates": [],
                        },
                    },
                ]

        request = RequestFactory().get("/test/ical")
        view = TestTimezoneFeed()

        response = view(request)
        calendar = icalendar.Calendar.from_ical(response.content)
        self.assertEqual(len(calendar.subcomponents), 2)

        self.assertEqual(
            calendar.subcomponents[0]["DTSTART"].to_ical(), b"20120501T180000"
        )
        self.assertEqual(
            calendar.subcomponents[0]["DTSTART"].params["TZID"], "JST"
        )

        self.assertEqual(
            calendar.subcomponents[0]["DTEND"].to_ical(), b"20120501T200000"
        )
        self.assertEqual(
            calendar.subcomponents[0]["DTEND"].params["TZID"], "JST"
        )

        self.assertEqual(
            calendar.subcomponents[1]["DTSTART"].to_ical(), b"20120506T180000"
        )
        self.assertEqual(
            calendar.subcomponents[1]["DTSTART"].params["TZID"], "EDT"
        )

        self.assertEqual(
            calendar.subcomponents[1]["DTEND"].to_ical(), b"20120506T200000"
        )
        self.assertEqual(
            calendar.subcomponents[1]["DTEND"].params["TZID"], "EDT"
        )

    def test_file_name(self):
        request = RequestFactory().get("/test/ical")
        view = TestFilenameFeed()

        response = view(request)

        self.assertIn("Content-Disposition", response)
        self.assertEqual(
            response["content-disposition"], 'attachment; filename="123.ics"'
        )

    def test_file_type(self):
        request = RequestFactory().get("/test/ical")
        view = TestFilenameFeed()
        response = view(request)
        self.assertIn("Content-Type", response)
        self.assertEqual(
            response["content-type"],
            "text/calendar; charset=utf8",
        )

    def test_file_header(self):
        request = RequestFactory().get("/test/ical")
        view = TestFilenameFeed()
        response = view(request)
        header = b"BEGIN:VCALENDAR\r\nVERSION:2.0"
        self.assertTrue(response.content.startswith(header))



"""
Views for generating ical feeds.
"""

from datetime import datetime
from calendar import timegm
from inspect import signature

from django.http import HttpResponse, Http404
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.syndication.views import Feed
from django.utils.http import http_date

from django_ical import feedgenerator

__all__ = ("ICalFeed",)

# Extra fields added to the Feed object
# to support ical
FEED_EXTRA_FIELDS = ("method", "product_id", "timezone")

# Extra fields added to items (events) to
# support ical
ICAL_EXTRA_FIELDS = [
    "timestamp",  # dtstamp
    "created",  # created
    "start_datetime",  # dtstart
    "end_datetime",  # dtend
    "transparency",  # transp
    "location",  # location
    "geolocation",  # latitude;longitude
    "organizer",  # email, cn, and role
    "rrule",  # rrule
    "exrule",  # exrule
    "rdate",  # rdate
    "exdate",  # exdate
    "status",  # CONFIRMED|TENTATIVE|CANCELLED
    "attendee",  # list of attendees
    "valarm",  # list of icalendar.Alarm objects
]


class ICalFeed(Feed):
    """
    iCalendar Feed

    Existing Django syndication feeds

    :title: X-WR-CALNAME
    :description: X-WR-CALDESC
    :item_guid: UID
    :item_title: SUMMARY
    :item_description: DESCRIPTION
    :item_link: URL
    :item_updateddate: LAST-MODIFIED

    Extension fields

    :method: METHOD
    :timezone: X-WR-TIMEZONE
    :item_class: CLASS
    :item_timestamp: DTSTAMP
    :item_created: CREATED
    :item_start_datetime: DTSTART
    :item_end_datetime: DTEND
    :item_transparency: TRANSP
    :item_attendee: ATTENDEE
    :item_valarm: VALARM
    """

    feed_type = feedgenerator.DefaultFeed

    def __call__(self, request, *args, **kwargs):
        """
        Copied from django.contrib.syndication.views.Feed

        Supports file_name as a dynamic attr.
        """
        try:
            obj = self.get_object(request, *args, **kwargs)
        except ObjectDoesNotExist:
            raise Http404("Feed object does not exist.")

        feedgen = self.get_feed(obj, request)
        response = HttpResponse(content_type=feedgen.mime_type)

        if hasattr(self, "item_pubdate") or hasattr(self, "item_updateddate"):
            # if item_pubdate or item_updateddate is defined for the feed, set
            # header so as ConditionalGetMiddleware is able to send 304 NOT MODIFIED
            response["Last-Modified"] = http_date(
                timegm(feedgen.latest_post_date().utctimetuple())
            )
        feedgen.write(response, "utf-8")

        filename = self._get_dynamic_attr("file_name", obj)
        if filename:
            response["Content-Disposition"] = 'attachment; filename="%s"' % filename

        return response

    def _get_dynamic_attr(self, attname, obj, default=None):
        """
        Copied from django.contrib.syndication.views.Feed (v1.7.1)
        """
        try:
            attr = getattr(self, attname)
        except AttributeError:
            return default
        if callable(attr):
            num_args = len(signature(attr).parameters)
            if num_args == 0:
                return attr()
            if num_args == 1:
                return attr(obj)

            raise TypeError(
                "Number of arguments to _get_dynamic_attr needs to be 0 or 1"
            )
        return attr

    # NOTE: Not used by icalendar but required
    #       by the Django syndication framework.
    link = ""

    def method(self, obj):  # pylint: disable=unused-argument
        return "PUBLISH"

    def feed_extra_kwargs(self, obj):
        kwargs = {}
        for field in FEED_EXTRA_FIELDS:
            val = self._get_dynamic_attr(field, obj)
            if val:
                kwargs[field] = val
        return kwargs

    def item_timestamp(self, obj):  # pylint: disable=unused-argument
        return datetime.now()

    def item_extra_kwargs(self, item):
        kwargs = {}
        for field in ICAL_EXTRA_FIELDS:
            val = self._get_dynamic_attr("item_" + field, item)
            if val:
                kwargs[field] = val
        return kwargs



"""Utility functions to build calendar rules."""

from icalendar.prop import vRecur
from recurrence import serialize


def build_rrule(  # noqa
    count=None,
    interval=None,
    bysecond=None,
    byminute=None,
    byhour=None,
    byweekno=None,
    bymonthday=None,
    byyearday=None,
    bymonth=None,
    until=None,
    bysetpos=None,
    wkst=None,
    byday=None,
    freq=None,
):
    """
    Build rrule dictionary for vRecur class.

    :param count: int
    :param interval: int
    :param bysecond: int
    :param byminute: int
    :param byhour: int
    :param byweekno: int
    :param bymonthday: int
    :param byyearday: int
    :param bymonth: int
    :param until: datetime
    :param bysetpos: int
    :param wkst: str, two-letter weekday
    :param byday: weekday
    :param freq: str, frequency name ('WEEK', 'MONTH', etc)
    :return: dict
    """
    result = {}

    if count is not None:
        result["COUNT"] = count

    if interval is not None:
        result["INTERVAL"] = interval

    if bysecond is not None:
        result["BYSECOND"] = bysecond

    if byminute is not None:
        result["BYMINUTE"] = byminute

    if byhour is not None:
        result["BYHOUR"] = byhour

    if byweekno is not None:
        result["BYWEEKNO"] = byweekno

    if bymonthday is not None:
        result["BYMONTHDAY"] = bymonthday

    if byyearday is not None:
        result["BYYEARDAY"] = byyearday

    if bymonth is not None:
        result["BYMONTH"] = bymonth

    if until is not None:
        result["UNTIL"] = until

    if bysetpos is not None:
        result["BYSETPOS"] = bysetpos

    if wkst is not None:
        result["WKST"] = wkst

    if byday is not None:
        result["BYDAY"] = byday

    if freq is not None:
        if freq not in vRecur.frequencies:
            raise ValueError(
                "Frequency value should be one of: {0}".format(vRecur.frequencies)
            )
        result["FREQ"] = freq

    return result


def build_rrule_from_text(rrule_str):
    """Build an rrule from a serialzed RRULE string."""
    recurr = vRecur()
    return recurr.from_ical(rrule_str)


def build_rrule_from_recurrences_rrule(rule):
    """
    Build rrule dictionary for vRecur class from a django_recurrences rrule.

    django_recurrences is a popular implementation for recurrences in django.
    https://pypi.org/project/django-recurrence/
    this is a shortcut to interface between recurrences and icalendar.
    """
    line = serialize(rule)
    if line.startswith("RRULE:"):
        line = line[6:]
    return build_rrule_from_text(line)


def build_rrule_from_dateutil_rrule(rule):
    """
    Build rrule dictionary for vRecur class from a dateutil rrule.

    Dateutils rrule is a popular implementation of rrule in python.
    https://pypi.org/project/python-dateutil/
    this is a shortcut to interface between dateutil and icalendar.
    """
    lines = str(rule).splitlines()
    for line in lines:
        if line.startswith("DTSTART:"):
            continue
        if line.startswith("RRULE:"):
            line = line[6:]
        return build_rrule_from_text(line)



import os
import sys


def main():
    """
    Standalone Django model test with a 'memory-only-django-installation'.
    You can play with a django model without a complete django app installation.
    http://www.djangosnippets.org/snippets/1044/
    """

    import django
    from django.conf import settings

    settings.configure(
        DATABASES={
            "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}
        },
        INSTALLED_APPS=["django.contrib.contenttypes", "django_ical"],
        MIDDLEWARE_CLASSES=[
            "django.middleware.common.CommonMiddleware",
            "django.contrib.sessions.middleware.SessionMiddleware",
        ],
        SECRET_KEY="snakeoil",
        TIME_ZONE="UTC",
    )

    django.setup()

    from django.test.utils import get_runner

    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(["django_ical"])
    sys.exit(bool(failures))


if __name__ == "__main__":
    main()



"""
iCalendar feed generation library -- used for generating
iCalendar feeds.

Sample usage:

>>> from django_ical import feedgenerator
>>> from datetime import datetime
>>> feed = feedgenerator.ICal20Feed(
...     title="My Events",
...     link="http://www.example.com/events.ical",
...     description="A iCalendar feed of my events.",
...     language="en",
... )
>>> feed.add_item(
...     title="Hello",
...     link="http://www.example.com/test/",
...     description="Testing."
...     start_datetime=datetime(2012, 5, 6, 10, 00),
...     end_datetime=datetime(2012, 5, 6, 12, 00),
... )
>>> fp = open('test.ical', 'w')
>>> feed.write(fp, 'utf-8')
>>> fp.close()

For definitions of the iCalendar format see:
http://www.ietf.org/rfc/rfc2445.txt
"""

from icalendar import Calendar, Event

from django.utils.feedgenerator import SyndicationFeed

__all__ = ("ICal20Feed", "DefaultFeed")

FEED_FIELD_MAP = (
    ("product_id", "prodid"),
    ("method", "method"),
    ("title", "x-wr-calname"),
    ("description", "x-wr-caldesc"),
    ("timezone", "x-wr-timezone"),
    (
        "ttl",
        "x-published-ttl",
    ),  # See format here: http://www.rfc-editor.org/rfc/rfc2445.txt (sec 4.3.6)
)

ITEM_EVENT_FIELD_MAP = (
    # 'item_guid' becomes 'unique_id' when passed to the SyndicationFeed
    ("unique_id", "uid"),
    ("title", "summary"),
    ("description", "description"),
    ("start_datetime", "dtstart"),
    ("end_datetime", "dtend"),
    ("updateddate", "last-modified"),
    ("created", "created"),
    ("timestamp", "dtstamp"),
    ("transparency", "transp"),
    ("location", "location"),
    ("geolocation", "geo"),
    ("link", "url"),
    ("organizer", "organizer"),
    ("categories", "categories"),
    ("rrule", "rrule"),
    ("exrule", "exrule"),
    ("rdate", "rdate"),
    ("exdate", "exdate"),
    ("status", "status"),
    ("attendee", "attendee"),
    ("valarm", None),
)


class ICal20Feed(SyndicationFeed):
    """
    iCalendar 2.0 Feed implementation.
    """

    mime_type = "text/calendar; charset=utf8"

    def write(self, outfile, encoding):  # pylint: disable=unused-argument
        """
        Writes the feed to the specified file in the
        specified encoding.
        """
        cal = Calendar()
        cal.add("version", "2.0")
        cal.add("calscale", "GREGORIAN")

        for ifield, efield in FEED_FIELD_MAP:
            val = self.feed.get(ifield)
            if val is not None:
                cal.add(efield, val)

        self.write_items(cal)

        to_ical = getattr(cal, "as_string", None)
        if not to_ical:
            to_ical = cal.to_ical
        outfile.write(to_ical())

    def write_items(self, calendar):
        """
        Write all events to the calendar
        """
        for item in self.items:
            event = Event()
            for ifield, efield in ITEM_EVENT_FIELD_MAP:
                val = item.get(ifield)
                if val is not None:
                    if ifield == "attendee":
                        for list_item in val:
                            event.add(efield, list_item)
                    elif ifield == "valarm":
                        for list_item in val:
                            event.add_component(list_item)
                    else:
                        event.add(efield, val)
            calendar.add_component(event)


DefaultFeed = ICal20Feed



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name="django-ical",
    description="iCal feeds for Django based on Django's syndication feed framework.",
    long_description="\n".join(
        [
            open("README.rst", encoding="utf-8").read(),
            open("CHANGES.rst", encoding="utf-8").read(),
        ]
    ),
    keywords="ical calendar django syndication feed",
    author="Ian Lewis",
    author_email="security@jazzband.com",
    maintainer="Jazzband",
    maintainer_email="security@jazzband.com",
    license="MIT License",
    url="https://github.com/jazzband/django-ical",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Environment :: Plugins",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=["django>=2.2", "icalendar>=4.0.3", "django-recurrence>=1.11.1"],
    packages=find_packages(),
    test_suite="tests.main",
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
)
